/*
 * Copyright (c) 2005 - 2007, Tranql project contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.tranql.connector.sqlserver2000;

import java.sql.SQLException;

import javax.resource.ResourceException;
import javax.resource.spi.ResourceAdapterInternalException;
import javax.security.auth.Subject;
import javax.sql.XAConnection;

import org.tranql.connector.CredentialExtractor;
import org.tranql.connector.NoExceptionsAreFatalSorter;
import org.tranql.connector.jdbc.AbstractXADataSourceMCF;

import com.microsoft.jdbcx.sqlserver.SQLServerDataSource;

/**
 * ManagedConnectionFactory that wraps a DB2 EmbeddedXADataSource for use in-VM.
 *
 * @version $Revision: 595 $ $Date: 2009-01-22 12:16:04 -0800 (Thu, 22 Jan 2009) $
 */
public class XAMCF extends AbstractXADataSourceMCF {
    private final SQLServerDataSource ds;
    private String password = "";

    /**
     * Default constructor for a DB2 Embedded XA DataSource.
     */
    public XAMCF() {
        super(new SQLServerDataSource(), new NoExceptionsAreFatalSorter());
        ds = (SQLServerDataSource) xaDataSource;
    }
    
    public void setDatabaseName(String dbname) {
        ds.setDatabaseName(dbname);
    }

    public String getDatabaseName() {
        return ds.getDatabaseName();
    }

    public void setPassword(String pwd) {
        this.password = pwd;
        ds.setPassword(pwd);
    }

    public String getPassword() {
        return password;
    }

    public void setPortNumber(Integer pn) {
        ds.setPortNumber(pn == null ? 0 : pn.intValue());
    }

    public Integer getPortNumber() {
        return new Integer(ds.getPortNumber());
    }

    public void setServerName(String sn) {
        ds.setServerName(sn);
    }

    public String getServerName() {
        return ds.getServerName();
    }

    public void setUserName(String user) {
        ds.setUser(user);
    }

    public String getUserName() {
        return ds.getUser();
    }
    
    public void setSelectMethod(String method) {
    	ds.setSelectMethod(method);
    }
    
    public String getSelectMethod() {
    	return ds.getSelectMethod();
    }
    
    public void setSendStringParameters(Boolean value) {
    	ds.setSendStringParametersAsUnicode(value.booleanValue());
    }
    
    public Boolean getSendStringParameters() {
    	return new Boolean( ds.getSendStringParametersAsUnicode() );
    }

    /**
     * Equality is defined in terms of the serverName, portNumber and databaseName properties
     */
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (obj instanceof XAMCF) {
        	SQLServerDataSource other = ((XAMCF) obj).ds;
            return equals(ds.getServerName(), other.getServerName()) &&
                    ds.getPortNumber() == other.getPortNumber() &&
                    equals(ds.getDatabaseName(), other.getDatabaseName());
        } else {
            return false;
        }
    }

    private static boolean equals(String a, String b) {
        return a == b || a != null && a.equals(b);
    }

    public int hashCode() {
        return hashCode(ds.getServerName()) ^ ds.getPortNumber() ^ hashCode(ds.getDatabaseName());
    }

    private static int hashCode(String s) {
        return s == null ? 0 : s.hashCode();
    }

    public String toString() {
        return "XAMCF[" + getDatabaseName() + ']';
    }
}
